import { toFormValidator } from '@vee-validate/zod';
import * as zod from 'zod';
import { SearchProvider, Options } from '~/components/inputs/DropdownInput.vue';
import { apiCall } from './api';

const GetHardwareInterfaces: SearchProvider = async (o) => {
  let res = await apiCall("Network.GetLinks", {});
  if (res.Error === null) {
    console.debug("links", res.Data.Links);
    return Object.fromEntries(res.Data.Links.map(r => [r.name, { display: r.name }]));
  } else {
    console.debug("error", res);
    return {} as Options;
  }
};

const GetInterfaces: SearchProvider = async (o) => {
  let res = await apiCall("Network.GetInterfaces", {});
  if (res.Error === null) {
    console.debug("interfaces", res.Data.Interfaces);
    let obj = {} as Options;
    Object.keys(res.Data.Interfaces).forEach(function (key, index) {
      obj[key] = {
        display: key,
      };
    });
    return obj;
  } else {
    console.debug("error", res);
    return {} as Options;
  }
};

const GetAddresses: SearchProvider = async (o) => {
  let res = await apiCall("Object.GetAddresses", {});
  if (res.Error === null) {
    console.debug("addresses", res.Data.Addresses);
    let obj = {} as Options;
    Object.keys(res.Data.Addresses).forEach(function (key, index) {
      obj[key] = {
        display: key,
      };
    });
    return obj;
  } else {
    console.debug("error", res);
    return {} as Options;
  }
};

const GetServices: SearchProvider = async (o) => {
  let res = await apiCall("Object.GetServices", {});
  if (res.Error === null) {
    console.debug("services", res.Data.Services);
    let obj = {} as Options;
    Object.keys(res.Data.Services).forEach(function (key, index) {
      obj[key] = {
        display: key,
      };
    });
    return obj;
  } else {
    console.debug("error", res);
    return {} as Options;
  }
};

const GetPeers: SearchProvider = async (o) => {
  let res = await apiCall("VPN.GetWireguardPeers", {});
  if (res.Error === null) {
    console.debug("peers", res.Data.WireguardPeers);
    let obj = {} as Options;
    Object.keys(res.Data.WireguardPeers).forEach(function (key, index) {
      obj[key] = {
        display: key,
      };
    });
    return obj;
  } else {
    console.debug("error", res);
    return {} as Options;
  }
};

export const editTypes: { [key: string]: { [key: string]: any } } = {
  "firewall": {
    name: "Firewall",
    "forwardrules": {
      name: "ForwardRule",
      validationSchema: toFormValidator(
        zod.object({
          name: zod.string(),
          verdict: zod.string(),
          counter: zod.boolean(),
          comment: zod.string().optional(),
        }),
      ),
      sections: [
        {
          fields: [
            { key: "name", label: "Name", as: "TextBox" },
            { key: "verdict", label: "Verdict", as: "PillBar", props: { options: { accept: { display: 'Accept' }, drop: { display: 'Drop' }, continue: { display: 'Continue' } } } },
            { key: "counter", label: "Counter", as: "CheckBox" },
            { key: "comment", label: "Comment", as: "MultilineTextBox" },
          ],
        },
      ],
    },
  },
  "network": {
    name: "Network",
    "interfaces": {
      name: "Interface",
      validationSchema: toFormValidator(
        zod.object({
          name: zod.string(),
          type: zod.string(),
          hardware_interface: zod.string().optional(),
          vlan_id: zod.number().optional(),
          comment: zod.string().optional(),
        }),
      ),
      sections: [
        {
          fields: [
            { key: "name", label: "Name", as: "TextBox", default: "placeholder" },
            { key: "type", label: "Type", as: "PillBar", props: { options: { hardware: { display: 'Hardware' }, vlan: { display: 'VLAN' }, bond: { display: 'Bond' }, bridge: { display: 'Bridge' } } } },
            { key: "hardware_device", label: "Hardware Device", as: "SingleSelect", enabled: (values: any) => (values["type"] == 'hardware'), props: { searchProvider: GetHardwareInterfaces } },
            { key: "vlan_parent", label: "VLAN Parent", as: "SingleSelect", enabled: (values: any) => (values["type"] == 'vlan'), props: { searchProvider: GetInterfaces } },
            { key: "vlan_id", label: "VLAN ID", as: "NumberBox", props: { min: 1, max: 4094 }, enabled: (values: any) => (values["type"] == 'vlan') },
            { key: "bond_members", label: "Bond Members", as: "MultiSelect", enabled: (values: any) => (values["type"] == 'bond'), props: { searchProvider: GetHardwareInterfaces } },
            { key: "bridge_members", label: "Bridge Members", as: "MultiSelect", enabled: (values: any) => (values["type"] == 'bridge'), props: { searchProvider: GetHardwareInterfaces } },
          ],
        },
        {
          title: "Addressing",
          fields: [
            { key: "addressing_mode", label: "Addressing Mode", as: "PillBar", props: { options: { none: { display: 'None' }, static: { display: 'Static' }, dhcp: { display: 'DHCP' } } } },
            { key: "address", label: "Address", as: "TextBox", enabled: (values: any) => (values["addressing_mode"] == 'static') },
            { key: "comment", label: "Comment", as: "MultilineTextBox" },
          ],
        },
      ],
    },
    "staticroutes": {
      name: "StaticRoute",
      validationSchema: toFormValidator(
        zod.object({
          name: zod.string(),
        }),
      ),
      sections: [
        {
          fields: [
            { key: "name", label: "Name", as: "TextBox" },
            { key: "interface", label: "Interface", as: "SingleSelect", props: { searchProvider: GetInterfaces } },
            { key: "gateway", label: "Gateway", as: "TextBox" },
            { key: "destination", label: "Destination", as: "TextBox" },
            { key: "metric", label: "Metric", as: "NumberBox" },
          ],
        },
      ],
    },
  },
  "object": {
    name: "Object",
    "addresses": {
      name: "Address",
      validationSchema: toFormValidator(
        zod.object({
        }),
      ),
      sections: [
        {
          fields: [
            { key: "name", label: "Name", as: "TextBox", default: "placeholder" },
            { key: "type", label: "Type", as: "PillBar", props: { options: { host: { display: 'Host' }, range: { display: 'Range' }, network: { display: 'Network' }, group: { display: 'Group' } } } },
            { key: "host", label: "Host", as: "TextBox", default: "placeholder", enabled: (values: any) => (values["type"] == 'host') },
            { key: "range", label: "Range", as: "TextBox", default: "placeholder", enabled: (values: any) => (values["type"] == 'range') },
            { key: "network", label: "Network", as: "TextBox", default: "placeholder", enabled: (values: any) => (values["type"] == 'network') },
            { key: "children", label: "Children", as: "MultiSelect", enabled: (values: any) => (values["type"] == 'group'), props: { searchProvider: GetAddresses } },
            { key: "comment", label: "Comment", as: "MultilineTextBox" },
          ],
        },
      ],
    },
    "services": {
      name: "Service",
      validationSchema: toFormValidator(
        zod.object({
        }),
      ),
      sections: [
        {
          fields: [
            { key: "name", label: "Name", as: "TextBox", default: "placeholder" },
            { key: "type", label: "Type", as: "PillBar", props: { options: { tcp: { display: 'TCP' }, udp: { display: 'UDP' }, icmp: { display: 'ICMP' }, group: { display: 'Group' } } } },
            { key: "sport_start", label: "Source Port Start", as: "NumberBox", enabled: (values: any) => (values["type"] == 'tcp' || values["type"] == 'udp') },
            { key: "sport_end", label: "Source Port End", as: "NumberBox", enabled: (values: any) => (values["type"] == 'tcp' || values["type"] == 'udp') },
            { key: "dport_start", label: "Destination Port Start", as: "NumberBox", enabled: (values: any) => (values["type"] == 'tcp' || values["type"] == 'udp') },
            { key: "dport_end", label: "Destination Port End", as: "NumberBox", enabled: (values: any) => (values["type"] == 'tcp' || values["type"] == 'udp') },
            { key: "icmp_code", label: "ICMP Code", as: "NumberBox", enabled: (values: any) => (values["type"] == 'icmp') },
            { key: "children", label: "Children", as: "MultiSelect", enabled: (values: any) => (values["type"] == 'group'), props: { searchProvider: GetServices } },
            { key: "comment", label: "Comment", as: "MultilineTextBox" },
          ],
        },
      ],
    },
  },
  "service": {
    name: "Service",
    "dhcpv4servers": {
      name: "DHCPv4Server",
      validationSchema: toFormValidator(
        zod.object({
        }),
      ),
      sections: [
        {
          fields: [
            { key: "interface", label: "Interface", as: "SingleSelect", props: { searchProvider: GetInterfaces } },
            { key: "pool", label: "Pool", as: "MultiSelect", props: { searchProvider: GetAddresses } },
            { key: "gateway_mode", label: "Gateway Mode", as: "PillBar", props: { options: { none: { display: 'None' }, interface: { display: 'Interface' }, specify: { display: 'Specify' } } } },
            { key: "gateway", label: "Gateway", as: "SingleSelect", enabled: (values: any) => (values["gateway_mode"] == 'specify'), props: { searchProvider: GetAddresses } },
            { key: "dns_server_mode", label: "DNS Server Mode", as: "PillBar", props: { options: { none: { display: 'None' }, interface: { display: 'Interface' }, specify: { display: 'Specify' } } } },
            { key: "dns_servers", label: "DNS Servers", as: "MultiSelect", enabled: (values: any) => (values["dns_server_mode"] == 'specify'), props: { searchProvider: GetAddresses } },
            { key: "ntp_server_mode", label: "NTP Server Mode", as: "PillBar", props: { options: { none: { display: 'None' }, interface: { display: 'Interface' }, specify: { display: 'Specify' } } } },
            { key: "ntp_servers", label: "NTP Servers", as: "MultiSelect", enabled: (values: any) => (values["ntp_server_mode"] == 'specify'), props: { searchProvider: GetAddresses } },
            { key: "default_lease_time", label: "Default Lease Time", as: "NumberBox" },
            { key: "max_lease_time", label: "Max Lease Time", as: "NumberBox" },
            { key: "comment", label: "Comment", as: "MultilineTextBox" },
          ],
        },
      ],
    },
    "ntpservers": {
      name: "NTPServer",
      validationSchema: toFormValidator(
        zod.object({
        }),
      ),
      sections: [
        {
          fields: [
            { key: "interface", label: "Interface", as: "SingleSelect", props: { searchProvider: GetInterfaces } },
            { key: "comment", label: "Comment", as: "MultilineTextBox" },
          ],
        },
      ],
    },
    "dnsservers": {
      name: "DNSServer",
      validationSchema: toFormValidator(
        zod.object({
        }),
      ),
      sections: [
        {
          fields: [
            { key: "interface", label: "Interface", as: "SingleSelect", props: { searchProvider: GetInterfaces } },
            { key: "comment", label: "Comment", as: "MultilineTextBox" },
          ],
        },
      ],
    },
  },
  "vpn": {
    name: "VPN",
    "wireguardinterfaces": {
      name: "WireguardInterface",
      validationSchema: toFormValidator(
        zod.object({
        }),
      ),
      sections: [
        {
          fields: [
            { key: "name", label: "Name", as: "TextBox", default: "placeholder" },
            { key: "public_key", label: "Public Key", as: "TextBox", default: "placeholder" },
            { key: "private_key", label: "Private Key", as: "TextBox", default: "placeholder" },
            { key: "listen_port", label: "Listen Port", as: "NumberBox" },
            { key: "peers", label: "Peers", as: "MultiSelect", props: { searchProvider: GetPeers } },
            { key: "comment", label: "Comment", as: "MultilineTextBox" },
          ],
        },
      ],
    },
    "wireguardpeers": {
      name: "WireguardPeer",
      validationSchema: toFormValidator(
        zod.object({
        }),
      ),
      sections: [
        {
          fields: [
            { key: "name", label: "Name", as: "TextBox", default: "placeholder" },
            { key: "public_key", label: "Public Key", as: "TextBox", default: "placeholder" },
            { key: "preshared_key", label: "Preshared Key", as: "TextBox", default: "placeholder" },
            { key: "allowed_ips", label: "Allowed IPs", as: "MultiSelect", props: { searchProvider: GetAddresses } },
            { key: "endpoint", label: "Endpoint", as: "TextBox", default: "placeholder" },
            { key: "persistent_keepalive", label: "Persistent Keepalive", as: "NumberBox" },
            { key: "comment", label: "Comment", as: "MultilineTextBox" },
          ],
        },
      ],
    },
  },
};